@@ -14,9 +14,19 @@
  *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include "defines.h"
+/* Some functions that didn't fit */
+
+#ifndef DEFINES_H
+# define DEFINES_H
+# include "defines.h"
+#endif
+
 #include "extra.h"
 #include "bits.h"
+#include "locks.h"
+
+extern char* outfile;
+extern int stream_flag;
 
 /* expands the one keyword to three (for tripleDES)
  */
@@ -39,23 +49,23 @@ char *tmpkey=key;
 
           for (i=0;i<3;i++) {
 
-              bzero(key_buf[i],sizeof(key_buf[i]));
+              Bzero(key_buf[i],sizeof(key_buf[i]));
               if (i==2) { /* last round */
-                memcpy(key_buf[i],tmpkey,D+y);
+                memmove(key_buf[i],tmpkey,D+y);
               }
               else {
-               memcpy(key_buf[i],tmpkey,D);    
+               memmove(key_buf[i],tmpkey,D);    
               }
          
               for (j=0;j<D;j++) {
                 for (z=0;z<(keylen-j-1);z++)
-                  memcpy(&tmpkey[z],&tmpkey[z+1],1);
+                  memmove(&tmpkey[z],&tmpkey[z+1],1);
               }
           }
 
-         memcpy(keyword1,key_buf[0],8); 
-         memcpy(keyword2,key_buf[1],8); 
-         memcpy(keyword3,key_buf[2],8); 
+         memmove(keyword1,key_buf[0],8); 
+         memmove(keyword2,key_buf[1],8); 
+         memmove(keyword3,key_buf[2],8); 
          return 0;
 }
 
@@ -86,7 +96,7 @@ char *btmp;
 
    atmp=getpass(prt);
 
-   bzero(ztmp,sizeof(ztmp));
+   Bzero(ztmp,sizeof(ztmp));
    strncat(ztmp,atmp,127);
 
    btmp=getpass("Re-Enter key: ");
@@ -112,7 +122,7 @@ char msg[100];
 /*   if (strncmp(tmp,"\0",1)==0) return NULL; */
    if(tmp==NULL) return NULL;
 
-   if ((strlen(tmp) > numofchar) && (type!=BLOWFISH)) { fprintf(stderr,_("Warning: keyword is too small for this algorithm\n")); }
+   if ((strlen(tmp) < numofchar) && (type!=BLOWFISH)) { fprintf(stderr,_("Warning: keyword is too small for this algorithm\n")); }
    if (strlen(tmp) > numofchar) { fprintf(stderr,_("Warning: Reducing the size of keyword\n")); }
  }
  else { /* 7bit mode */
@@ -143,6 +153,8 @@ int show_mode(int ende, int stype,int smode,char *output,int bitmode)
   if (stype==ThreeWAY) fprintf (stderr,_( "Algorithm: 3-WAY\n"));
   if (stype==BLOWFISH) fprintf (stderr,_( "Algorithm: Blowfish\n"));
   if (stype==GOST) fprintf (stderr,_( "Algorithm: GOST\n"));
+  if (stype==SAFER64) fprintf (stderr,_( "Algorithm: SAFER64\n"));  
+  if (stype==SAFER128) fprintf (stderr,_( "Algorithm: SAFER128\n"));
   if (stype==CRYPT) {
     fprintf (stderr,_( "Algorithm: Unix Crypt\n"));  
   }
@@ -166,7 +178,7 @@ return 0;
 }
 
 
-#ifdef HAVE_SYS_STAT_H
+#ifdef HAVE_STAT
 
 int check_file(char * filename) {
 struct stat ostat;
@@ -183,23 +195,6 @@ struct stat ostat;
 
 #endif
 
-int my_read(int fd, void *buf, size_t count) {
-
-  int i,x;
-  char *tmpbuf=buf;
-
- x=read(fd,tmpbuf,count);
- if (x < count) { 
-  for (i=x;i<count;i++) {
-    x=read(fd,&tmpbuf[i],1);
-    if (x==0) return i;
-    if (x==-1) return (-1);
-  }
- }
- 
-return count;
-
-}
 
 int check_file_head(char * filename, unsigned int *algorithm, unsigned int *mode, unsigned int *bit7_flag, unsigned int stream_flag)
 {
@@ -211,6 +206,7 @@ char buf[6];
  }
  else {
   fstream=fopen(filename,"r");
+  read_lock( fileno((FILE*)fstream));
  } 
 
  fread(buf, 1, 6, fstream);
@@ -224,12 +220,14 @@ char buf[6];
 
 /* 6 To 9 are crc32 */
 
-  if (stream_flag==FALSE) fclose(fstream);
+  if (stream_flag==FALSE) { fflush(fstream); unlock( fileno((FILE*)fstream)); fclose(fstream); }
   return 0;
  }  
  else { /* No headers present */
    if (stream_flag==FALSE) { 
-    fclose(fstream); 
+    fflush(fstream);
+    unlock( fileno((FILE*)fstream));
+    fclose(fstream);
     return 1;
    }
    else { /* restart the stream so we will not loose important data */
@@ -270,8 +268,8 @@ unsigned char buf[6];
    
 }
 
-#ifdef HAVE_SYS_STAT_H
-# ifdef HAVE_UTIME_H
+#ifdef HAVE_STAT
+# ifdef HAVE_UTIME
 void copyDate ( char *srcName, char *dstName )
 {
    int retVal;
@@ -288,5 +286,43 @@ void copyDate ( char *srcName, char *dstName )
    if (retVal==-1) perror("utime");
 }
 # endif
+
+
+int is_normal_file(char *filename)
+{
+   struct stat statBuf;
+
+# ifdef HAVE_LSTAT /* Do not treat symlinks as regular files */
+   if (lstat ( filename, &statBuf )!=0) return FALSE;
+# else
+   if (stat ( filename, &statBuf )!=0) return FALSE;
+# endif
+
+   if ( S_ISREG(statBuf.st_mode)!=0 ) {
+     return TRUE;
+   }
+   else {
+     return FALSE;
+   }
+
+}
+
 #endif
-                                       
\ No newline at end of file
+
+void shandler(int signal)
+{
+
+ fprintf(stderr, _("Signal %d caught. Exiting.\n"),signal);
+ cleanUp();
+ exit(-1);
+
+}
+
+void cleanUp() {
+
+fflush(NULL);
+if (stream_flag==FALSE) remove(outfile); /* Delete the file we were writing to */
+
+
+}
+
