@@ -24,16 +24,13 @@
 */
 
 #include "mcrypt.h"
-#include "des.h"
-#include "gost.h"
-#include "3-way.h"
-#include "crypt.h"
 #include "blowfish.h"
 #include "bits.h"
 #include "lcrypt.h"
 #include "sunlink.h"
 #include "crc32.h"
 #include "locks.h"
+#include "crypt.h"
 
  unsigned int bit7_flag=BITS7;
  unsigned int stream_flag=FALSE;
@@ -57,7 +54,7 @@ void usage(char *name)
  -d --decrypt                 decrypts\n\
  -a --algorithm=ALGORITHM     specify the encryption and decryption algorithm.\n\
                               Legal parameters are: 3way, des, 3des, gost,\n\
-                              safer64, safer128, crypt and blowfish. \n\
+                              safer64, safer128, cast128, crypt and blowfish. \n\
                               (Default is blowfish)\n\
  -m --mode=MODE               specify the mode (ecb or cbc). (Default is cbc)\n\
  -k --key=KEYWORD             keyword to use.\n\
@@ -250,6 +247,11 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
                                   if (strncasecmp(typet,"SAFER128",8)==0) {
                                     type=SAFER128;
                                   }
+                                  else {
+                                    if (strncasecmp(typet,"CAST128",8)==0) {
+                                    type=CAST128;
+                                    }
+                                  }
                                 }
                               }
                              }
@@ -397,6 +399,7 @@ for(i=0;i<file_count;i++) {
 			case GOST:
 			case SAFER64:
 			case SAFER128:
+                        case CAST128:
 			 x=decrypt_general(type,dinfile,outfile,keyword);
 			 break;
 			case CRYPT:
@@ -444,6 +447,7 @@ for(i=0;i<file_count;i++) {
 	        	case GOST:
 	        	case SAFER64:
 	        	case SAFER128:
+                        case CAST128: 	        	
 	        	 x=encrypt_general(type,einfile,outfile,keyword);
 	        	 break;
 	        	 
@@ -614,6 +618,7 @@ FILE *FROMF;
 int keysize=get_key_size(algorithm);
 int blocksize=get_block_size(algorithm);
 char tmpkey[128];
+int td;
 
 word32 *IV=0;
 word32 *keyword=calloc(keysize,1); 
@@ -711,7 +716,8 @@ if (mode==CBC) {
     }
  }
 
- init_mcrypt(algorithm,keyword,len);
+ td = init_mcrypt(algorithm,keyword,len);
+
  if (bare_flag==FALSE) clear_crc32();
 
 
@@ -754,7 +760,7 @@ switch (mode) {
       }
 
 
-     mcrypt(algorithm, keyword, len, ciphertext);
+     mcrypt(td, ciphertext);
 
      /* for cbc keep the old ciphertext */
      memmove(ciphertextcbc,ciphertext,blocksize);
@@ -789,7 +795,7 @@ switch (mode) {
   /* crc32 */
    if (bare_flag==FALSE) crc32(ciphertext, how);
 
-   mcrypt(algorithm, keyword, len, ciphertext);
+   mcrypt(td, ciphertext);
 
          
     if (fwrite(ciphertext, 1, blocksize, TOF) == 0) {perror("fwrite"); return 1;}
@@ -817,7 +823,7 @@ switch (mode) {
    for(j=0; j<blocksize/sizeof(word32); j++)
      ciphertext[j] = RAND32;
    memmove(ciphertext,&fcrc32,sizeof(fcrc32));
-   mcrypt(algorithm, keyword, len, ciphertext);
+   mcrypt(td, ciphertext);
    fseek(TOF,6,SEEK_SET);
    if (fwrite(ciphertext, 1, blocksize, TOF) == 0) {perror("fwrite"); return 1;}
 
@@ -849,7 +855,7 @@ switch (mode) {
  }
                 
     fclose(TOF);
-    end_mcrypt(algorithm);
+    end_mcrypt(td);
   
     fclose(RTOF);
     fclose(FROMF);
@@ -883,6 +889,7 @@ char *tmp;
 int keysize=get_key_size(algorithm);
 int blocksize=get_block_size(algorithm);
 char tmpkey[128];
+int td;
 
 word32 *ciphertext_old=malloc(blocksize);
 word32 *buffer=malloc(blocksize);
@@ -964,7 +971,7 @@ if (quiet==FALSE) show_mode(DECRYPT,algorithm,mode,tofile,bit7_flag);
     }
  }
  
- init_mcrypt(algorithm,keyword,len);
+ td = init_mcrypt(algorithm,keyword,len);
 
 /* Decrypt CRC32 stored in file */
  if (bare_flag==FALSE) {
@@ -973,7 +980,7 @@ if (quiet==FALSE) show_mode(DECRYPT,algorithm,mode,tofile,bit7_flag);
    
     /* Read CRC32 */
    fread(ciphertext, 1, blocksize, FROMF);
-   mdecrypt(algorithm, keyword, len ,ciphertext);
+   mdecrypt(td, ciphertext);
    memmove(&fcrc32, ciphertext, sizeof(fcrc32));
 #ifndef WORDS_BIGENDIAN
    fcrc32=byteswap(fcrc32);
@@ -1010,7 +1017,7 @@ switch (mode) {
         memmove(ciphertext_orig,ciphertext,blocksize); /* keep backup */
 
 
-        mdecrypt(algorithm, keyword, len, ciphertext);
+        mdecrypt(td, ciphertext);
 	
   /* if cbc: xor the new plaintext with the old ciphertext */
         if (start>0) {
@@ -1082,7 +1089,7 @@ switch (mode) {
         memmove(ciphertext_orig,ciphertext,blocksize); /* keep backup */
 
 
-        mdecrypt(algorithm, keyword, len, ciphertext);
+        mdecrypt(td, ciphertext);
 	
         memmove(ciphertext_old,ciphertext_orig,blocksize);
 
@@ -1130,7 +1137,7 @@ switch (mode) {
 
 /* ready */
 
-  end_mcrypt(algorithm);  
+  end_mcrypt(td);  
 
   free(ciphertext_old);
   free(buffer);
