@@ -1,8 +1,6 @@
-#ifndef DEFINES_H_INC
- #define DEFINES_H_INC
+#ifndef DEFINES_H
+ #define DEFINES_H
  #include "defines.h"
 #endif
 
-#ifdef HAVE_SYS_STAT_H
 int safe_unlink(char * filename);
-#endif
